/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2011, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** newt interface                                                    Rev 0.0  11/21/2011 **/
/**                                                                                       **/
/*******************************************************************************************/
module interface (kc0, kc1, kc2, kc3, kc4, kc5, kc6, ser_rx, vci, vco, DATA_BUS_IN, DPWO_IN,
                  FI_BUS_IN, ISA_BUS_IN, KR0_IN, KR1_IN, KR2_IN, KR3_IN, KR4_IN, KR5_IN,
                  KR6_IN, KR7_IN, LLD_IN, MEMCS1B, MMU_EN, POR_IN, PWO_IN, SYNC_DRV,
                  UART_RX, clk, data_bus, dpwo, fi_bus, isa_bus, kr0, kr1, kr2, kr3,
                  kr4, kr5, kr6, kr7, lld, mem_cs1, por, pwo, pwr_up, ser_tx, KC0_HI, KC1_HI,
                  KC2_HI, KC3_HI, KC4_HI, KC5_HI, KC6_HI, MMU_RES, MMU_SET, PWO_RES, SYNC_HI,
                  UART_TX);

  input         clk;           /* master clock                                        osc  */
  input         data_bus;      /* memory/peripheral data bus                          mem  */
  input         dpwo;          /* display power on request                            bufr */
  input         fi_bus;        /* flag input bus                                      bufr */
  input         isa_bus;       /* instruction address/data bus                        bufr */
  input         kr0;           /* keyscan row 0                                       kbrd */
  input         kr1;           /* keyscan row 1                                       kbrd */
  input         kr2;           /* keyscan row 2                                       kbrd */
  input         kr3;           /* keyscan row 3                                       kbrd */
  input         kr4;           /* keyscan row 4                                       kbrd */
  input         kr5;           /* keyscan row 5                                       kbrd */
  input         kr6;           /* keyscan row 6                                       kbrd */
  input         kr7;           /* keyscan row 7                                       kbrd */
  input         lld;           /* low-level detect                                    brd  */
  input         mem_cs1;       /* memory chip select 1                                fpga */
  input         por;           /* power on request                                    kbrd */
  input         pwo;           /* power off                                           bufr */
  input         pwr_up;        /* power on init                                       brd  */
  input         ser_rx;        /* serial receive                                      brd  */
  input         KC0_HI;        /* keyscan column 0                                    fpga */
  input         KC1_HI;        /* keyscan column 1                                    fpga */
  input         KC2_HI;        /* keyscan column 2                                    fpga */
  input         KC3_HI;        /* keyscan column 3                                    fpga */
  input         KC4_HI;        /* keyscan column 4                                    fpga */
  input         KC5_HI;        /* keyscan column 5                                    fpga */
  input         KC6_HI;        /* keyscan column 6                                    fpga */
  input         MMU_RES;       /* disable mmu                                         fpga */
  input         MMU_SET;       /* enable mmu                                          fpga */
  input         PWO_RES;       /* pwo pulse                                           fpga */
  input         SYNC_HI;       /* synchronization output hi drive                     fpga */
  input         UART_TX;       /* uart tx data                                        fpga */

  output        kc0;           /* keyscan column 0                                    kbrd */
  output        kc1;           /* keyscan column 1                                    kbrd */
  output        kc2;           /* keyscan column 2                                    kbrd */
  output        kc3;           /* keyscan column 3                                    kbrd */
  output        kc4;           /* keyscan column 4                                    kbrd */
  output        kc5;           /* keyscan column 5                                    kbrd */
  output        kc6;           /* keyscan column 6                                    kbrd */
  output        ser_tx;        /* serial transmit                                     kbrd */
  output        vci;           /* vcc request                                         brd  */
  output        vco;           /* vcc acknowledge (master reset)                      fpga */
  output        DATA_BUS_IN;   /* memory/peripheral data bus                          fpga */
  output        DPWO_IN;       /* display power on request                            fpga */
  output        FI_BUS_IN;     /* flag input bus                                      fpga */
  output        ISA_BUS_IN;    /* instruction address/data bus                        fpga */
  output        KR0_IN;        /* keyscan row 0                                       fpga */
  output        KR1_IN;        /* keyscan row 1                                       fpga */
  output        KR2_IN;        /* keyscan row 2                                       fpga */
  output        KR3_IN;        /* keyscan row 3                                       fpga */
  output        KR4_IN;        /* keyscan row 4                                       fpga */
  output        KR5_IN;        /* keyscan row 5                                       fpag */
  output        KR6_IN;        /* keyscan row 6                                       fpga */
  output        KR7_IN;        /* keyscan row 7                                       fpga */
  output        LLD_IN;        /* low-level detect                                    fpga */
  output        MEMCS1B;       /* memory chip select 1                                mem  */
  output        MMU_EN;        /* mmu enable                                          fpga */
  output        POR_IN;        /* power on request                                    fpga */
  output        PWO_IN;        /* power off input                                     fpga */
  output        SYNC_DRV;      /* sync drive                                          bufr */
  output        UART_RX;       /* uart rx data                                        fpga */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        deep_slp, key_strt, light_slp, por_strt, ser_tx;
  wire        kc0, kc1, kc2, kc3, kc4, kc5, kc6, kc_en, kc_val, vco;
  wire        DATA_BUS_IN, DPWO_IN, FI_BUS_IN, ISA_BUS_IN, LLD_IN, PWO_IN, SYNC_DRV;
  wire        KR0_IN, KR1_IN, KR2_IN, KR3_IN, KR4_IN, KR5_IN, KR6_IN, KR7_IN, POR_IN;
  wire        MEMCS1B, MMU_EN, UART_RX;

  reg         dpwo_reg, mmu_en, oen_reg, stable_reg, vci, vco_lat;
  reg   [2:0] vco_reg;
  reg  [13:0] delay_reg;

  /*****************************************************************************************/
  /*                                                                                       */
  /* keyboard interface                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  assign kc_en  = light_slp ||
                  (KC0_HI && KC1_HI && KC2_HI && KC3_HI && KC4_HI && KC5_HI && KC6_HI);
  assign kc_val = !light_slp &&
                   KC0_HI && KC1_HI && KC2_HI && KC3_HI && KC4_HI && KC5_HI && KC6_HI;

  assign kc0    = (kc_en || !KC0_HI) ? kc_val : 1'bz;      /* key column 0                 */
  assign kc1    = (kc_en || !KC1_HI) ? kc_val : 1'bz;      /* key column 1                 */
  assign kc2    = (kc_en || !KC2_HI) ? kc_val : 1'bz;      /* key column 2                 */
  assign kc3    = (kc_en || !KC3_HI) ? kc_val : 1'bz;      /* key column 3                 */
  assign kc4    = (kc_en || !KC4_HI) ? kc_val : 1'bz;      /* key column 4                 */
  assign kc5    = (kc_en || !KC5_HI) ? kc_val : 1'bz;      /* key column 5                 */
  assign kc6    = (kc_en || !KC6_HI) ? kc_val : 1'bz;      /* key column 6                 */

  assign KR0_IN = kr0 && stable_reg;                       /* key row 0 input              */
  assign KR1_IN = kr1 && stable_reg;                       /* key row 1 input              */
  assign KR2_IN = kr2 && stable_reg;                       /* key row 2 input              */
  assign KR3_IN = kr3 && stable_reg;                       /* key row 3 input              */
  assign KR4_IN = kr4 && stable_reg;                       /* key row 4 input              */
  assign KR5_IN = kr5 && stable_reg;                       /* key row 5 input              */
  assign KR6_IN = kr6 && stable_reg;                       /* key row 6 input              */
  assign KR7_IN = kr7 && stable_reg;                       /* key row 7 input              */
  assign POR_IN = por && stable_reg;                       /* key row por input            */

  /*****************************************************************************************/
  /*                                                                                       */
  /* FPGA inputs                                                                           */
  /*                                                                                       */
  /*****************************************************************************************/
  assign DATA_BUS_IN = data_bus && stable_reg;             /* data bus                     */
  assign DPWO_IN     = dpwo     && stable_reg;             /* from display                 */
  assign FI_BUS_IN   = fi_bus   && stable_reg;             /* flag input                   */
  assign ISA_BUS_IN  = isa_bus  && stable_reg;             /* isa bus                      */
  assign LLD_IN      = lld      && stable_reg;             /* low level detect             */
  assign MMU_EN      = mmu_en   && stable_reg;             /* mmu enable                   */
  assign PWO_IN      = oen_reg;                            /* cpu output enable            */
  assign UART_RX     = ser_rx   && stable_reg;             /* uart rx data                 */

  /*****************************************************************************************/
  /*                                                                                       */
  /* FPGA outputs                                                                          */
  /*                                                                                       */
  /*****************************************************************************************/
  assign MEMCS1B     = !mem_cs1 || !vco_lat || !oen_reg;   /* memory chip select 1         */
  assign ser_tx      = UART_TX  || !oen_reg;               /* serial tx data               */
  assign SYNC_DRV    = (SYNC_HI && oen_reg) || light_slp;  /* synchronization              */

  /*****************************************************************************************/
  /*                                                                                       */
  /* power up/down control                                                                 */
  /*                                                                                       */
  /*****************************************************************************************/
  assign deep_slp  = !pwo && !dpwo;
  assign light_slp = !pwo &&  dpwo;

  assign key_strt  = dpwo && !(por && kr7 && kr6 && kr5 && kr4 && kr3 && kr2 && kr1 && kr0);
  assign por_strt  = !por || isa_bus;

  always @ (posedge clk or negedge vci) begin
    if (!vci) begin
      delay_reg  <= 14'h0000;
      oen_reg    <= 1'b0;
      stable_reg <= 1'b0;
      end
    else begin
      delay_reg  <=  delay_reg + !stable_reg;
      oen_reg    <= &delay_reg[13:8] || oen_reg;
      stable_reg <= &delay_reg || stable_reg;
      end
    end

  always @ (posedge clk or negedge pwr_up) begin
    if                (!pwr_up) mmu_en <= 1'b0;
    else if (stable_reg && pwo) mmu_en <= !MMU_RES && (MMU_SET || mmu_en);
    end

  always @ (por_strt or pwr_up or deep_slp) begin
    if (!pwr_up || (deep_slp && !por_strt)) vci <= 1'b0;
    else if                     (por_strt)  vci <= 1'b1;
    end

  always @ (posedge clk or negedge pwr_up) begin
    if (!pwr_up) vco_lat <= 1'b0;
    else         vco_lat <= stable_reg && !PWO_RES &&
                            (isa_bus || key_strt || !dpwo_reg || vco_lat);
    end

  always @ (negedge clk or negedge pwr_up) begin
    if (!pwr_up) begin
      dpwo_reg <= 1'b0;
      vco_reg  <= 3'b000;
      end
    else begin
      dpwo_reg <= dpwo;
      vco_reg  <= {(vco_lat && (&vco_reg[1:0] || vco_reg[2])),
                   (vco_lat && (^vco_reg[1:0] || vco_reg[2])),
                   (vco_lat && (!vco_reg[0]   || vco_reg[2]))};
      end
    end

  assign vco = vco_reg[2];

  endmodule







