/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2013, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** keyboard scanner                                                  Rev 0.0  03/07/2013 **/
/**                                                                                       **/
/*******************************************************************************************/
module kbrd_top (kc0, kc1, kc2, kc3, kc4, kc5, kc6, ky_data, ky_flag, chkkb_inst, kr0, kr1,
                 kr2, kr3, kr4, kr5, kr6, kr7, ld_cycle, ld_inst, ph2, por, rstb, rstkb_inst,
                 te_ky);

  input         chkkb_inst;    /* chk kb instruction                                       */
  input         kr0;           /* keyscan row 0                                            */
  input         kr1;           /* keyscan row 1                                            */
  input         kr2;           /* keyscan row 2                                            */
  input         kr3;           /* keyscan row 3                                            */
  input         kr4;           /* keyscan row 4                                            */
  input         kr5;           /* keyscan row 5                                            */
  input         kr6;           /* keyscan row 6                                            */
  input         kr7;           /* keyscan row 7                                            */
  input         ld_cycle;      /* load machine cycle                                       */
  input         ld_inst;       /* load instruction                                         */
  input         ph2;           /* processor clock                                          */
  input         por;           /* power on request                                         */
  input         rstb;          /* master reset                                             */
  input         rstkb_inst;    /* rst kb instruction                                       */
  input         te_ky;         /* time enable: nibbles 3,4                                 */
  output        kc0;           /* keyscan column 0                                         */
  output        kc1;           /* keyscan column 1                                         */
  output        kc2;           /* keyscan column 2                                         */
  output        kc3;           /* keyscan column 3                                         */
  output        kc4;           /* keyscan column 4                                         */
  output        kc5;           /* keyscan column 5                                         */
  output        kc6;           /* keyscan column 6                                         */
  output        ky_flag;       /* keyboard flag                                            */
  output        ky_data;       /* keyboard data                                            */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        col_same;                                    /* key still down same col      */
  wire        kc0, kc1, kc2, kc3, kc4, kc5, kc6;           /* kbrd column drive            */
  wire        keytime;                                     /* kbrd scan active             */
  wire        ky_data;                                     /* keyboard data                */
  wire        ky_flag;                                     /* keyboard flag                */
  wire  [3:0] keycol_nxt;                                  /* kbrd column sequence         */

  reg         key_two;                                     /* two keys down                */
  reg         keyblnk_reg;                                 /* leaving kbrd state 3         */
  reg         keyclr_reg;                                  /* kbrd state (2 or 3)          */
  reg         keyhold_reg;                                 /* kbrd state (3)               */
  reg         keyrun_reg;                                  /* running (not sleep) state    */
  reg         keysame_reg;                                 /* key still down same key      */
  reg         keys2_reg;                                   /* latched two keys down        */
  reg         prechar_reg;                                 /* precharge                    */
  reg         row_same;                                    /* key still down same row      */
  reg   [3:0] keycol_reg;                                  /* kbrd column scan             */
  reg   [3:0] key_row;                                     /* kbrd row encode              */
  reg   [7:0] keyscan_reg;                                 /* latched key code             */
  reg   [7:0] ky_reg;                                      /* keyboard register            */
  reg   [8:0] keysmpl_reg;                                 /* kbrd row sampled             */

  /*****************************************************************************************/
  /*                                                                                       */
  /* keyboard scanner                                                                      */
  /*                                                                                       */
  /*****************************************************************************************/
  assign keycol_nxt = {((!keycol_reg[3] &&  keycol_reg[2]) ||
                        ( keycol_reg[3] && !keycol_reg[0])),
                       ((!keycol_reg[2] &&  keycol_reg[1]) ||
                        ( keycol_reg[3] && !keycol_reg[0])),
                       ((!keycol_reg[2] &&  keycol_reg[0]) ||
                        ( keycol_reg[2] && !keycol_reg[0])),
                       ((!keycol_reg[2] &&  keycol_reg[0]) ||
                        ( keycol_reg[1] && !keycol_reg[0]))};

  always @ (posedge ph2 or negedge rstb) begin
    if (!rstb) begin
      keyrun_reg <= 1'b0;
      keycol_reg <= 4'b0000;
      end
    else if (ld_cycle) begin
      keyrun_reg <=  ld_inst || keyrun_reg;
      keycol_reg <= (ld_inst) ? 4'b0001 : keycol_nxt;
      end
    end

  assign kc0 = (!(!keycol_reg[1] &&  keycol_reg[0]) && keyrun_reg) || prechar_reg;
  assign kc1 =  !(!keycol_reg[2] &&  keycol_reg[1]) || prechar_reg;
  assign kc2 =  !(!keycol_reg[3] &&  keycol_reg[2]) || prechar_reg;
  assign kc3 =  !( keycol_reg[3] && !keycol_reg[2]) || prechar_reg;
  assign kc4 =  !( keycol_reg[2] && !keycol_reg[1]) || prechar_reg;
  assign kc5 =  !( keycol_reg[1] && !keycol_reg[0]) || prechar_reg;
  assign kc6 =  !( keycol_reg[3] &&  keycol_reg[0]) || prechar_reg;

  assign keytime = |keycol_reg;

  always @ (posedge ph2 or negedge rstb) begin
    if        (!rstb) keysmpl_reg <= 9'b111111111;
    else if (keytime) keysmpl_reg <= {por, kr7, kr6, kr5, kr4, kr3, kr2, kr1, kr0};
    end

  always @ (keysmpl_reg) begin
    casex (keysmpl_reg)
      9'b0xxxxxxxx: key_row = 4'b1000;
      9'b10xxxxxxx: key_row = 4'b0111;
      9'b110xxxxxx: key_row = 4'b0110;
      9'b1110xxxxx: key_row = 4'b0101;
      9'b11110xxxx: key_row = 4'b0100;
      9'b111110xxx: key_row = 4'b0011;
      9'b1111110xx: key_row = 4'b0010;
      9'b11111110x: key_row = 4'b0001;
      9'b111111110: key_row = 4'b0000;
      default:      key_row = 4'b1111;
      endcase
    end

  always @ (posedge ph2 or negedge rstb) begin
    if                                    (!rstb) keyscan_reg <= 8'b00000000;
    else if (ld_cycle && keytime && !keyhold_reg) keyscan_reg <= {keycol_reg, key_row};
    end

  always @ (posedge ph2 or negedge rstb) begin
    if                 (!rstb) ky_reg <= 8'b00000000;
    else if (ld_inst || te_ky) ky_reg <= (ld_inst) ? keyscan_reg : {ky_reg[0], ky_reg[7:1]};
    end

  assign ky_data = keyhold_reg && ky_reg[0];
  assign ky_flag = keyhold_reg && !keyclr_reg;

  /*****************************************************************************************/
  /*                                                                                       */
  /* keyboard state machine                                                                */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2 or negedge rstb) begin
    if (!rstb) begin
      keyhold_reg <= 1'b0; 
      keyclr_reg  <= 1'b0;
      keyblnk_reg <= 1'b0;
      prechar_reg <= 1'b0;
      end
    else begin
      keyhold_reg <= (ld_cycle && keytime && ~&key_row && !keyblnk_reg) || 
                     (!(ld_inst && chkkb_inst && keyclr_reg) && keyhold_reg);
      keyclr_reg  <= (ld_inst && rstkb_inst && !keysame_reg && !keys2_reg && keyhold_reg) ||
                     (!(ld_inst && chkkb_inst) && keyclr_reg);
      if (ld_inst) keyblnk_reg <= chkkb_inst && keyhold_reg && keyclr_reg;
      prechar_reg <= ld_cycle && keyrun_reg;
      end
    end

  always @ (keyscan_reg or keysmpl_reg) begin
    case (keyscan_reg[3:0])
      4'b1000: row_same = !keysmpl_reg[8];
      4'b0111: row_same = !keysmpl_reg[7];
      4'b0110: row_same = !keysmpl_reg[6];
      4'b0101: row_same = !keysmpl_reg[5];
      4'b0100: row_same = !keysmpl_reg[4];
      4'b0011: row_same = !keysmpl_reg[3];
      4'b0010: row_same = !keysmpl_reg[2];
      4'b0001: row_same = !keysmpl_reg[1];
      4'b0000: row_same = !keysmpl_reg[0];
      default: row_same = 1'b0;
      endcase
    end

  assign col_same = ~|(keycol_reg ^ keyscan_reg[7:4]);

  always @ (keysmpl_reg) begin
    case (keysmpl_reg)
      9'b011111111,
      9'b101111111,
      9'b110111111,
      9'b111011111,
      9'b111101111,
      9'b111110111,
      9'b111111011,
      9'b111111101,
      9'b111111110,
      9'b111111111: key_two = 1'b0;
      default:      key_two = 1'b1;
      endcase
    end

  always @ (posedge ph2 or negedge rstb) begin
    if (!rstb) begin
      keysame_reg <= 1'b0;
      keys2_reg   <= 1'b0;
      end
    else if (ld_cycle) begin
      keysame_reg <= !ld_inst && ((col_same && row_same && keytime) || keysame_reg);
      keys2_reg   <= !ld_inst && ((key_two && keytime) || keys2_reg);
    end
    end

  endmodule







