/*******************************************************************************************/
/**                                                                                       **/
/** COPYRIGHT (C) 2014, Systemyde International Corporation, ALL RIGHTS RESERVED          **/
/**                                                                                       **/
/** newt processor                                                    Rev 0.0  02/26/2014 **/
/**                                                                                       **/
/*******************************************************************************************/
module newt_top (MEMDATA, MEMCS0B, mem_cs1, MEMRDB, MEMWRB, DATA_BUS_EN, DATA_BUS_HI,
                 FO_BUS_HI, ISA_BUS_EN, ISA_BUS_HI, KC0_HI, KC1_HI, KC2_HI, KC3_HI, KC4_HI,
                 KC5_HI, KC6_HI, MEMADDR, MMU_RES, MMU_SET, PDAT, PH1_HI, PH2_HI, PPLS,
                 PWO_HI, PWO_RES, SDIO_CEB, SDIO_SCK, SDIO_DO, SDIO_WPB, SYNC_HI, UART_TX,
                 CLK_IN, DATA_BUS_IN, DPWO_IN, FI_BUS_IN, ISA_BUS_IN, KR0_IN, KR1_IN, KR2_IN,
                 KR3_IN, KR4_IN, KR5_IN, KR6_IN, KR7_IN, LLD_IN, MMU_EN, MODE41, POR_IN,
                 PWO_IN, SDIO_DI, UART_RX, VCO_IN);

  inout  [15:0] MEMDATA;       /* memory data bus                                     mem  */
  input         CLK_IN;        /* master clock                                        brd  */
  input         DATA_BUS_IN;   /* memory/peripheral data bus                          cpld */
  input         DPWO_IN;       /* display power on request                            cpld */
  input         FI_BUS_IN;     /* flag input bus                                      cpld */
  input         ISA_BUS_IN;    /* instruction address/data bus                        cpld */
  input         KR0_IN;        /* keyscan row 0                                       cpld */
  input         KR1_IN;        /* keyscan row 1                                       cpld */
  input         KR2_IN;        /* keyscan row 2                                       cpld */
  input         KR3_IN;        /* keyscan row 3                                       cpld */
  input         KR4_IN;        /* keyscan row 4                                       cpld */
  input         KR5_IN;        /* keyscan row 5                                       cpld */
  input         KR6_IN;        /* keyscan row 6                                       cpld */
  input         KR7_IN;        /* keyscan row 7                                       cpld */
  input         LLD_IN;        /* low-level detect                                    cpld */
  input         MMU_EN;        /* mmu enable                                          cpld */
  input         MODE41;        /* 41c mode                                            brd  */
  input         POR_IN;        /* power on request                                    cpld */
  input         PWO_IN;        /* power off input                                     cpld */
  input         SDIO_DI;       /* sdio di                                             brd  */
  input         UART_RX;       /* uart rx data                                        cpld */
  input         VCO_IN;        /* vcc acknowledge (master reset)                      cpld */
  output        MEMCS0B;       /* memory chip select 0                                mem  */
  output        mem_cs1;       /* memory chip select 1                                cpld */
  output        MEMRDB;        /* memory read                                         mem  */
  output        MEMWRB;        /* memory write                                        mem  */
  output        DATA_BUS_EN;   /* memory/peripheral data bus enable                   bufr */
  output        DATA_BUS_HI;   /* memory/peripheral data bus hi drive                 bufr */
  output        FO_BUS_HI;     /* flag output bus hi drive                            bufr */
  output        ISA_BUS_EN;    /* instruction address/data bus enable                 bufr */
  output        ISA_BUS_HI;    /* instruction address/data bus hi drive               bufr */
  output        KC0_HI;        /* keyscan column 0 hi drive                           cpld */
  output        KC1_HI;        /* keyscan column 1 hi drive                           cpld */
  output        KC2_HI;        /* keyscan column 2 hi drive                           cpld */
  output        KC3_HI;        /* keyscan column 3 hi drive                           cpld */
  output        KC4_HI;        /* keyscan column 4 hi drive                           cpld */
  output        KC5_HI;        /* keyscan column 5 hi drive                           cpld */
  output        KC6_HI;        /* keyscan column 6 hi drive                           cpld */
  output        MMU_RES;       /* disable mmu                                         cpld */
  output        MMU_SET;       /* enable mmu                                          cpld */
  output        PH1_HI;        /* clock phase 1 hi drive                              bufr */
  output        PH2_HI;        /* clock phase 2 hi drive                              bufr */
  output        PPLS;          /* parallel data strobe                                brd  */
  output        PWO_HI;        /* power off                                           bufr */
  output        PWO_RES;       /* power off RESET                                     cpld */
  output        SDIO_CEB;      /* sdio ce#                                            brd  */
  output        SDIO_DO;       /* sdio do                                             brd  */
  output        SDIO_SCK;      /* sdio clk                                            brd  */
  output        SDIO_WPB;      /* sdio wp#                                            brd  */
  output        SYNC_HI;       /* synchronization output hi drive                     cpld */
  output        UART_TX;       /* uart tx data                                        clpd */
  output  [7:0] PDAT;          /* parallel data                                       brd  */
  output [22:0] MEMADDR;       /* memory address bus                                  mem  */

  /*****************************************************************************************/
  /*                                                                                       */
  /* signal declarations                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  wire        chkkb_inst;                                  /* chk kb instruction           */
  wire        cld_regadd;                                  /* combined load reg address    */
  wire        clk;                                         /* master clock                 */
  wire        clk2en;                                      /* enable clk2 (cpu)            */
  wire        clr_bank;                                    /* clear bank selects           */
  wire        data_bus_e, data_bus_h;                      /* data output predrive         */
  wire        data_drv;                                    /* data direction (cpu)         */
  wire        data_in;                                     /* data input (cpu)             */
  wire        data_out;                                    /* data output (cpu)            */
  wire        fo_bus;                                      /* flag output                  */
  wire        hsync_cpu;                                   /* hold boundary (cpu)          */
  wire        hsync_ext;                                   /* hold boundary (ext)          */
  wire        isa_bus_e, isa_bus_h;                        /* isa output predrive          */
  wire        isa_drv;                                     /* isa output enable (cpu)      */
  wire        isa_drv_en;                                  /* isa sample enable (cpu, clk) */
  wire        isa_in;                                      /* inst add/data in (cpu)       */
  wire        isa_nxt;                                     /* next inst add/data out (cpu) */
  wire        isa_out;                                     /* inst add/data out (cpu)      */
  wire        isa_recir;                                   /* recirculate address          */
  wire        isync_both;                                  /* inst boundary (both)         */
  wire        isync_cpu;                                   /* inst boundary (cpu)          */
  wire        isync_ext;                                   /* inst boundary (ext)          */
  wire        jmpldi_reg;                                  /* jmp/jsb/ldi/cxisa latched    */
  wire        kc0, kc1, kc2, kc3, kc4, kc5, kc6;           /* kbrd column drives           */
  wire        ky_data;                                     /* keyboard data                */
  wire        ky_flag;                                     /* keyboard flag                */
  wire        ld_bank;                                     /* load bank select             */
  wire        ld_35bank, ld_89bank, ld_abbank;             /* individual load bank         */
  wire        ld_cdbank, ld_efbank;
  wire        ld_4bank,  ld_6bank,  ld_7bank;
  wire        ld_cycle;                                    /* next cycle (cpu)             */
  wire        ld_cyclex;                                   /* next cycle (ext)             */
  wire        ld_divrate;                                  /* load rate divider            */
  wire        ld_inst;                                     /* load instruction (cpu)       */
  wire        ld_instx;                                    /* load instruction (ext)       */
  wire        ld_ldata;                                    /* load memory data latch       */
  wire        ld_mdata0, ld_mdata1;                        /* load ph mem data             */
  wire        ld_mdata2, ld_mdata3;
  wire        ld_minst;                                    /* load ph mem instruction      */
  wire        ld_mmmu;                                     /* load ph mem mmu              */
  wire        ld_mmucp;                                    /* load mmu cpld bit            */
  wire        ld_mmudis;                                   /* clear mmu enable latch       */
  wire        ld_mmuen;                                    /* set mmu enable latch         */
  wire        ld_mosdis;                                   /* clear map OS latch           */
  wire        ld_mosen;                                    /* set map OS latch             */
  wire        ld_mphase;                                   /* load mem st mach phase       */
  wire        ld_mword;                                    /* load mem word                */
  wire        ld_parport;                                  /* load parallel port           */
  wire        ld_pdata;                                    /* load peripheral data         */
  wire        ld_pfad;                                     /* load peripheral address      */
  wire        ld_turbo;                                    /* load turbo register          */
  wire        ld_usctrl;                                   /* load uart/sdio control       */
  wire        m_strt;                                      /* start phy mem state mach     */
  wire        mdata_shft;                                  /* reg data shift enable        */
  wire        mdata0_shft, mdata1_shft;                    /* reg data shift enables       */
  wire        mdata2_shft, mdata3_shft;
  wire        mem_cs1;                                     /* memory cs1 drive             */
  wire        mem_idl;                                     /* drive mem_data idle          */
  wire        mem_wr;                                      /* drive mem_data               */
  wire        minst_shft;                                  /* inst shift enable            */
  wire        mmuen_global;                                /* mmu global enable            */
  wire        mreg_out;                                    /* register serial data         */
  wire        page_ext, page_012, page_35;                 /* rom page address decodes     */
  wire        page_89, page_ab, page_cd, page_ef;
  wire        page5_cur;                                   /* mod page 5 during fetch      */
  wire        page5_ref;                                   /* mod page 5 during refetch    */
  wire        ph2c;                                        /* phase 2 (cpu)                */
  wire        ph2x;                                        /* phase 2 (ext)                */
  wire        powoff_inst;                                 /* power off instruction        */
  wire        r41_eany;                                    /* 41c register map hole        */
  wire        r41_eblk;                                    /* 41c register map hole        */
  wire        rd_mdata;                                    /* read ph mem data             */
  wire        reg_41;                                      /* 41c register map             */
  wire        reg_access;                                  /* register memory access       */
  wire        reg_read;                                    /* register memory read         */
  wire        reg_write;                                   /* register memory write        */
  wire        resetb;                                      /* master reset                 */
  wire        rstkb_inst;                                  /* rst kb instruction           */
  wire        run_1x;                                      /* cpu run 1x                   */
  wire        run_sync;                                    /* cpu clock enable (1x)        */
  wire        s_clk_out;                                   /* sdio clock output            */
  wire        s_data_in;                                   /* sdio data input              */
  wire        s_data_out;                                  /* sdio data output             */
  wire        selprf_reg;                                  /* selprf latched               */
  wire        statex_b;                                    /* decoded state (ext)          */
  wire        sync_mux;                                    /* sync output (newt)           */
  wire        sync_recir;                                  /* recirculate instruction      */
  wire        sync_raw;                                    /* raw sync output (cpu)        */
  wire        sync_reg;                                    /* sync output (cpu)            */
  wire        tagged_ref;                                  /* tagged for refetch           */
  wire        tagged_1x;                                   /* tagged for 1x execution      */
  wire        te_a14;                                      /* time enable: addr 14         */
  wire        tex_a14;                                     /* time enable: addr 14 (ext)   */
  wire        tex_ky;                                      /* time enable: nibl 3-4 (ext)  */
  wire        trbrun_res;                                  /* reset turbo operation        */
  wire        trbrun_set;                                  /* set turbo operation          */
  wire        trbhfs_set;                                  /* set turbo hold for special   */
  wire        trbwfr_set;                                  /* set turbo wait for refetch   */
  wire        trbref_set;                                  /* set turbo reftech            */
  wire        trbwff_set;                                  /* set turbo wait for finish    */
  wire        trboff_set;                                  /* set turbo run 1x speed       */
  wire        trig_wradd, trig_irdreg, trig_erdreg;        /* ph mem start signals         */
  wire        trig_rdmmu, trig_rwlog,  trig_wrom;
  wire        trig_rwmmu, trig_rwphy,  trig_wrreg;
  wire        u_data_in;                                   /* uart rx data                 */
  wire        u_data_out;                                  /* uart tx data                 */
  wire        uld_regadd;                                  /* daddc load reg address       */
  wire        urd_regadd;                                  /* cregn load reg address       */
  wire        us_bufr_done;                                /* sio tx buffer done           */
  wire        us_bufr_empt;                                /* sio tx buffer empty          */
  wire        us_bufr_full;                                /* sio rx buffer full           */
  wire        us_bufr_ovr;                                 /* sio rx buffer overflow       */
  wire        us_bufr_rd;                                  /* sio rx buffer read           */
  wire        us_bufr_wr;                                  /* sio tx buffer write          */
  wire        uwr_regadd;                                  /* regnc load reg address       */
  wire  [3:0] cycle_reg;                                   /* clock cycle                  */
  wire  [3:0] reg_emod, reg_imod;                          /* modified register address    */
  wire  [3:0] state_reg;                                   /* machine state                */
  wire  [5:0] mword_nxt;                                   /* ph mem word sel              */
  wire  [6:0] mphase_nxt;                                  /* ph mem phase                 */
  wire  [7:0] us_data_out;                                 /* sio rx data                  */
  wire [11:0] pg5_addr;                                    /* page 5 address               */
  wire [11:0] reg_addr;                                    /* register address             */
  wire [15:0] mrd_data;                                    /* buffered memory read data    */
  wire [55:0] dout_reg;                                    /* stable data register         */

  wire        DATA_BUS_EN, DATA_BUS_HI;                    /* mem/periph data i/o pads     */
  wire        FO_BUS_HI;                                   /* flag output pad              */
  wire        ISA_BUS_EN, ISA_BUS_HI;                      /* inst add/data i/o pads       */
  wire        KC0_HI, KC1_HI, KC2_HI, KC3_HI;              /* kbrd column drive pads       */
  wire        KC4_HI, KC5_HI, KC6_HI;
  wire        MEMCS0B;                                     /* memory cs0 drive             */
  wire        MEMRDB;                                      /* memory rd drive              */
  wire        MEMWRB;                                      /* memory wr drive              */
  wire        MMU_RES;                                     /* reset mmu enable pad         */
  wire        MMU_SET;                                     /* set mmu enable pad           */
  wire        PH1_HI;                                      /* phase 1 output pads          */
  wire        PH2_HI;                                      /* phase 2 output pads          */
  wire        PWO_HI;                                      /* power on output pad          */
  wire        SYNC_HI;                                     /* synchronization output pads  */
  wire [15:0] MEMDATA;                                     /* memory data bus              */
  wire [22:0] MEMADDR;                                     /* memory address bus           */

  reg         awake_reg;                                   /* awake state                  */
  reg         cdata_inst;                                  /* c=data inststruction         */
  reg         chkkb_instc;                                 /* chk kb instruction (clk)     */
  reg         cregn_inst;                                  /* c=regn inststruction         */
  reg         daddc_inst;                                  /* dadd=c instruction           */
  reg         data_pad;                                    /* ph1 sampled data input       */
  reg         data_ftch;                                   /* data=c or c=data inst        */
  reg         datac_inst;                                  /* data=c instruction           */
  reg         ckdivc_en;                                   /* decoded clock div            */
  reg         ckdivc_pls;                                  /* turbo clock pulse en         */
  reg         ckdivm_rst;                                  /* reset clock divider msb      */
  reg         ckph1_pls;                                   /* ph1 clock pulse en           */
  reg         cks_mode;                                    /* clocked serial mode          */
  reg         dftch_reg;                                   /* latched c=data/data=c        */
  reg         dpwo;                                        /* ph1 sampled disp pwr ctl     */
  reg         enbank_inst;                                 /* enbank instruction           */
  reg         fi_bus;                                      /* ph1 sampled flag input       */
  reg         goslp_reg;                                   /* go to sleep                  */
  reg         isa_drv_sync;                                /* isa drv (cpu, clk)           */
  reg         isax_drv_sync;                               /* isa drv (ext, clk)           */
  reg         isa_pad;                                     /* ph1 sampled isa              */
  reg         isax_drv;                                    /* isa output enable (ext)      */
  reg         kbrd_inst;                                   /* chkkb/rstkb instruction      */
  reg         kr0, kr1, kr2, kr3, kr4, kr5, kr6;           /* ph1 sampled kbrd row inputs  */
  reg         kr7, por;
  reg         ky_datac;                                    /* keyboard data reg (clk)      */
  reg         ky_flagc;                                    /* keyboard flag reg (clk)      */
  reg         ld_inst_dly;                                 /* inst start pulse (newt, dly) */
  reg         ld_instc;                                    /* inst start pulse (clk)       */
  reg         ld_instc_dly;                                /* inst start pulse (clk, dly)  */
  reg         ld_instxc;                                   /* inst start pulse (newt, clk) */
  reg         lld;                                         /* ph1 sampled low batt detect  */
  reg         lmmudis_reg;                                 /* latched mmu disable cmd      */
  reg         lmmuen_reg;                                  /* latched mmu enable cmd       */
  reg         mmmuen_reg;                                  /* mmu translate enable         */
  reg         osmap_reg;                                   /* OS map enable                */
  reg         par_dpls;                                    /* parallel data pulse          */
  reg         pfads_reg;                                   /* pfad=0xf0 selected           */
  reg         pfadc_inst;                                  /* pfad=c instruction           */
  reg         ph1;                                         /* phase 1 output               */
  reg         ph2;                                         /* phase 2 output               */
  reg         powoff_instc;                                /* powoff instruction (clk)     */
  reg         pwo;                                         /* power on output              */
  reg         pwo_pls;                                     /* pwo timing pulse             */
  reg         pwo_strobe;                                  /* pwo timing pulse for ext     */
  reg         regnc_inst;                                  /* regn=c inststruction         */
  reg         rftch_ftch;                                  /* 1x speed inst (refetch)      */
  reg         rread_ftch;                                  /* register memory read         */
  reg         rdtype_reg;                                  /* register read type           */
  reg         rstkb_instc;                                 /* rst kb instruction (clk)     */
  reg         rstsyn_reg;                                  /* reset synchronizer           */
  reg         s_ce;                                        /* sdio ce                      */
  reg         s_reset;                                     /* sdio synchronous reset       */
  reg         s_wp;                                        /* sdio wp                      */
  reg         slow_ftch;                                   /* 1x speed inst                */
  reg         state_1c, state_4c, state_ac;                /* decoded state (clk)          */
  reg         state_8c, state_9c, state_bc;
  reg         state_32c, statex_32c, statex_ec;
  reg         sync_rawc;                                   /* raw sync (clk)               */
  reg         syncx_reg;                                   /* sync output (ext)            */
  reg         syncx_regc;                                  /* sync output (ext, clk)       */
  reg         trbrun_reg;                                  /* turbo mode running           */
  reg         trbhfs_reg;                                  /* turbo hold for special       */
  reg         trbwfr_reg;                                  /* turbo wait for refetch       */
  reg         trbref_reg;                                  /* turbo refetch                */
  reg         trbwff_reg;                                  /* turbo wait for finish        */
  reg         trboff_reg;                                  /* turbo run 1x speed           */
  reg         wcmd_inst;                                   /* wcmd instruction             */
  reg         wr_mmudis;                                   /* clear cpld mmu enable latch  */
  reg         wr_mmuen;                                    /* set cpld mmu enable latch    */
  reg         wrom_inst;                                   /* wrom instruction             */
  reg   [1:0] addr_smp;                                    /* sampled cpu address output   */
  reg   [1:0] bank_sel;                                    /* rom bank select              */
  reg   [1:0] bank4_reg;                                   /* rom page 4 bank select       */
  reg   [1:0] bank5_reg;                                   /* rom page 5 bank select       */
  reg   [1:0] bank6_reg;                                   /* rom page 6 bank select       */
  reg   [1:0] bank7_reg;                                   /* rom page 7 bank select       */
  reg   [1:0] bank89_reg;                                  /* rom page 8/9 bank select     */
  reg   [1:0] bankab_reg;                                  /* rom page a/b bank select     */
  reg   [1:0] bankcd_reg;                                  /* rom page c/d bank select     */
  reg   [1:0] bankef_reg;                                  /* rom page e/f bank select     */
  reg   [1:0] mcycle_reg;                                  /* mem state machine cycle      */
  reg   [1:0] mspeci_reg;                                  /* ph mem special inst          */
  reg   [1:0] startup_reg;                                 /* startup sequence             */
  reg   [2:0] rstdly_reg;                                  /* reset startup delay          */
  reg   [2:0] slow_reg;                                    /* 1x operation delay           */
  reg   [2:0] turbo_reg, turbox_reg;                       /* turbo mode                   */
  reg   [3:0] cyclex_reg;                                  /* clock cycle (ext)            */
  reg   [3:0] diradd_reg;                                  /* direct address               */
  reg   [3:0] statex_reg;                                  /* machine state (ext)          */
  reg   [3:0] ckdivm_reg;                                  /* clock divider msb            */
  reg   [3:0] mword_sel;                                   /* register word select         */
  reg   [3:0] state_regc;                                  /* synced cpu state             */
  reg   [4:1] ckdivl_reg;                                  /* clock divider lsb            */
  reg   [5:0] mword_reg;                                   /* mem state machine word       */
  reg   [6:0] mphase_reg;                                  /* mem state machine phase      */
  reg   [7:0] mreg3_reg;                                   /* physical memory data 55:48   */
  reg   [9:0] ftch_reg;                                    /* latched isa data             */
  reg   [9:0] lftch_reg;                                   /* stable isa data              */
  reg   [9:0] minst_reg;                                   /* physical memory instruction  */
  reg  [11:0] div_rate;                                    /* uart baud rate divider       */
  reg  [11:0] mmmu_reg;                                    /* physical memory mmu data     */
  reg  [11:0] regadd_reg;                                  /* latched register address     */
  reg  [11:0] trans_addr;                                  /* translated address ms byte   */
  reg  [15:0] addr_reg;                                    /* latched isa address          */
  reg [15:12] laddr_reg;                                   /* stable isa address           */
  reg  [15:0] ldata_reg;                                   /* latched memory data          */
  reg  [15:0] mwr_data;                                    /* buffered memory write data   */
  reg  [15:0] mreg0_reg;                                   /* physical memory data 15:0    */
  reg  [15:0] mreg1_reg;                                   /* physical memory data 31:16   */
  reg  [15:0] mreg2_reg;                                   /* physical memory data 47:32   */
  reg  [15:0] preg_reg;                                    /* peripheral shifter           */
  reg  [15:0] uart_data;                                   /* uart peri data               */
  reg  [23:0] mem_addr;                                    /* memory address bus drive     */

  /*****************************************************************************************/
  /*                                                                                       */
  /* output buffers                                                                        */
  /*                                                                                       */
  /*****************************************************************************************/
  assign DATA_BUS_EN = (data_bus_e || (ph2 && pwo));
  assign DATA_BUS_HI = (data_bus_h && !ph2);
  assign FO_BUS_HI   =  fo_bus;
  assign ISA_BUS_EN  = (isa_bus_e || (ph2 && pwo));
  assign ISA_BUS_HI  = (isa_bus_h && !ph2);
  assign KC0_HI      =  kc0;
  assign KC1_HI      =  kc1;
  assign KC2_HI      =  kc2;
  assign KC3_HI      =  kc3;
  assign KC4_HI      =  kc4;
  assign KC5_HI      =  kc5;
  assign KC6_HI      =  kc6;
  assign MEMADDR     =  mem_addr[22:0];
  assign MEMDATA     = (mem_idl || mem_wr) ? mwr_data : 16'bzzzzzzzzzzzzzzzz;
  assign MMU_RES     =  wr_mmudis;
  assign MMU_SET     =  wr_mmuen;
  assign PDAT        = {turbo_reg, cks_mode, pfads_reg, selprf_reg, isax_drv, ld_instx};
  assign PPLS        =  par_dpls;
  assign PH1_HI      =  ph1;
  assign PH2_HI      =  ph2;
  assign PWO_HI      =  pwo;
  assign PWO_RES     =  goslp_reg;
  assign SDIO_CEB    = !s_ce;
  assign SDIO_DO     =  s_data_out;
  assign SDIO_SCK    =  s_clk_out;
  assign SDIO_WPB    = !s_wp;
  assign SYNC_HI     =  sync_mux;
  assign UART_TX     =  u_data_out;

  /*****************************************************************************************/
  /*                                                                                       */
  /* power control                                                                         */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clk or negedge resetb) begin
    if      (!resetb) pwo <= 1'b0;
    else if (pwo_pls) pwo <= !powoff_instc;
    end

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      awake_reg   <= 1'b0;
      goslp_reg   <= 1'b0;
      pwo_strobe  <= 1'b0;
      startup_reg <= 2'b00;
      end
    else begin
      awake_reg   <= !pwo_strobe && ((ld_instxc && pwo) || awake_reg);
      goslp_reg   <= !pwo && statex_ec && ckdivl_reg[4] && ~|(ckdivm_reg ^ 4'b0011);
      pwo_strobe  <=  pwo_pls && powoff_instc;
      startup_reg <= {!startup_reg[0], |startup_reg};
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* input buffers                                                                         */
  /*                                                                                       */
  /*****************************************************************************************/
  assign mmuen_global = MMU_EN;
  assign mrd_data     = MEMDATA;
  assign reg_41       = MODE41;
  assign resetb       = VCO_IN;
  assign s_data_in    = SDIO_DI;
  assign u_data_in    = UART_RX;

  /*****************************************************************************************/
  /*                                                                                       */
  /* external signal samples                                                               */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clk) begin
    if (ckph1_pls || &startup_reg) begin
      data_pad <=  DATA_BUS_IN;
      dpwo     <=  DPWO_IN;
      fi_bus   <= !FI_BUS_IN;
      isa_pad  <=  ISA_BUS_IN;
      kr0      <=  KR0_IN;
      kr1      <=  KR1_IN;
      kr2      <=  KR2_IN;
      kr3      <=  KR3_IN;
      kr4      <=  KR4_IN;
      kr5      <=  KR5_IN;
      kr6      <=  KR6_IN;
      kr7      <=  KR7_IN;
      lld      <=  LLD_IN;
      por      <=  POR_IN;
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* gated master clock                                                                    */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (negedge CLK_IN or negedge resetb) begin
    if (!resetb) rstdly_reg <= 3'b000;
    else         rstdly_reg <= {1'b1, rstdly_reg[2:1]};
    end

  always @ (negedge CLK_IN) begin
    rstsyn_reg <= rstdly_reg[0];
    end

  assign clk = CLK_IN && rstsyn_reg;

  /*****************************************************************************************/
  /*                                                                                       */
  /* turbo automatic speed shift control                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      ld_instc     <= 1'b0;
      ld_instc_dly <= 1'b0;
      ld_instxc    <= 1'b0;
      state_32c    <= 1'b0;
      statex_32c   <= 1'b0;
      statex_ec    <= 1'b0;
      syncx_regc   <= 1'b0;
      end
    else begin
      ld_instc     <= ld_inst;
      ld_instc_dly <= ld_instc && clk2en;
      ld_instxc    <= ld_instx;
      state_32c    <= ~|state_reg[3:2] && &state_reg[1:0] && cycle_reg[2];
      statex_32c   <= ~|statex_reg[3:2] && &statex_reg[1:0] && cyclex_reg[2];
      statex_ec    <= &statex_reg[3:1] && !statex_reg[0];
      syncx_regc   <= syncx_reg;
      end
    end

  always @ (posedge clk or negedge resetb) begin
    if        (!resetb) turbox_reg <= 3'b000;
    else if (awake_reg) turbox_reg <= turbo_reg;
    end

  assign hsync_cpu  = state_32c  && ckdivc_pls;
  assign hsync_ext  = statex_32c && ckdivm_rst;
  assign isync_both =  ld_instc  &&  ld_instxc  && ckdivm_rst;
  assign isync_cpu  =  ld_instc  && !ld_instxc  && ckdivc_pls;
  assign isync_ext  = !ld_instc  &&  ld_instxc  && ckdivm_rst;

  assign tagged_ref = mspeci_reg[1] || (rftch_ftch && !jmpldi_reg) || page_ext || selprf_reg;
  assign tagged_1x  = (!mspeci_reg[1] && mspeci_reg[0]) || (slow_ftch && !jmpldi_reg) ||
                      slow_reg[0];

  assign trbrun_res = trbrun_reg && ((hsync_cpu && tagged_ref) ||
                      ((isync_cpu || isync_both) && tagged_1x));
  assign trbrun_set = trboff_reg && isync_both && |turbox_reg && !(tagged_1x || tagged_ref);
  assign trbhfs_set = trbrun_reg && isync_cpu && tagged_1x;
  assign trbwfr_set = trbrun_reg && hsync_cpu && !isync_ext && tagged_ref;
  assign trbref_set = (trbwfr_reg && isync_ext) ||
                      (trbrun_reg && hsync_cpu && isync_ext && tagged_ref);
  assign trbwff_set = trbref_reg && hsync_ext;
  assign trboff_set = (trbhfs_reg && isync_ext) || (trbwff_reg && isync_both) ||
                      (trbrun_reg && isync_both && tagged_1x);

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      trbrun_reg <= 1'b0;
      trbhfs_reg <= 1'b0;
      trbwfr_reg <= 1'b0;
      trbref_reg <= 1'b0;
      trbwff_reg <= 1'b0;
      trboff_reg <= 1'b1;
      end
    else begin
      trbrun_reg <= trbrun_set || (!trbrun_res && trbrun_reg);
      trbhfs_reg <= trbhfs_set || (!isync_ext  && trbhfs_reg);
      trbwfr_reg <= trbwfr_set || (!isync_ext  && trbwfr_reg);
      trbref_reg <= trbref_set || (!hsync_ext  && trbref_reg);
      trbwff_reg <= trbwff_set || (!isync_both && trbwff_reg);
      trboff_reg <= trboff_set || (!trbrun_set && trboff_reg);
      end
    end

  assign data_in    = (reg_read)  ? mreg_out :
                      (pfads_reg) ? preg_reg[0]  : data_pad;
  assign isa_in     = (sync_raw && !page_ext) ? minst_reg[0] : isa_pad;
  assign run_1x     = trbwff_reg || trboff_reg;
  assign run_sync   = trbrun_reg || trbwff_reg || trboff_reg;
  assign sync_mux   = (run_1x) ? sync_reg : syncx_reg;
  assign sync_recir = run_1x && syncx_regc && !page_ext;

  assign isa_bus_e  = ((trbref_reg && isax_drv) || (run_1x && syncx_reg && !page_ext) ||
                       (trboff_reg && isa_drv)) && awake_reg;
  assign isa_bus_h  = (trbref_reg && isax_drv && addr_reg[0] && !page5_ref) ||
                      (run_1x && syncx_reg && minst_reg[0]) ||
                      (trboff_reg && isa_drv && isa_out && !page5_cur);

  assign r41_eany   = reg_41 && ((~|regadd_reg[11:5] && regadd_reg[4]) ||
                      (~|regadd_reg[11:6] && regadd_reg[5]) ||
                      (~|regadd_reg[11:10] && regadd_reg[9] && ~|regadd_reg[7:0]) ||
                      (~|regadd_reg[11:10] && regadd_reg[9] && &regadd_reg[7:4]));
  assign r41_eblk   = reg_41 && ((~|regadd_reg[11:5] && regadd_reg[4]) ||
                      (~|regadd_reg[11:6] && regadd_reg[5]) ||
                      (~|regadd_reg[11:10] && regadd_reg[9] && &regadd_reg[7:4]));

  assign reg_read   = !selprf_reg &&
                      ((cregn_inst && !r41_eblk) || (cdata_inst && !r41_eany));
  assign reg_write  = !selprf_reg &&
                      ((regnc_inst && !r41_eblk) || (datac_inst && !r41_eany));

  assign data_bus_h =  trboff_reg && (( reg_read && mreg_out) ||
                                      (!reg_read && data_out));
  assign data_bus_e = (!trboff_reg || reg_read || data_drv) && awake_reg;

  /*****************************************************************************************/
  /*                                                                                       */
  /* clock control                                                                         */
  /*                                                                                       */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   ckdivm_reg    00000111112222233333444445555566666777778888899999                    */
  /*   ckdivl_reg    01234012340123401234012340123401234012340123401234                    */
  /*   ckdivm_rst    .................................................1                    */
  /*   ph1           ................................111111111111......                    */
  /*   ph2           111111111111......................................                    */
  /*   ph2c (1x)     .................................................1                    */
  /*   ph2c (2x)     ........................1........................1                    */
  /*   ph2c (5x)     .........1.........1.........1.........1.........1                    */
  /*   ph2c (10x)    ....1....1....1....1....1....1....1....1....1....1                    */
  /*   ph2c (20x)    ..1.1..1.1..1.1..1.1..1.1..1.1..1.1..1.1..1.1..1.1                    */
  /*   ph2c (50x)    11111111111111111111111111111111111111111111111111                    */
  /*                                                                                       */
  /*****************************************************************************************/
  assign clk2en      = (ckdivm_rst && run_sync) || (ckdivc_pls && trbrun_reg);
  assign ph2c        = !clk && clk2en;
  assign ph2x        = !clk && ckdivm_rst;

  assign mdata_shft  = ckdivm_rst && run_sync && (cregn_inst || cdata_inst);
  assign mdata0_shft = clk2en && (cdata_inst || cregn_inst) && mword_sel[0];
  assign mdata1_shft = clk2en && (cdata_inst || cregn_inst) && mword_sel[1];
  assign mdata2_shft = clk2en && (cdata_inst || cregn_inst) && mword_sel[2];
  assign mdata3_shft = clk2en && (cdata_inst || cregn_inst) && mword_sel[3];

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      ckdivc_pls <= 1'b0;
      ckdivl_reg <= 4'b0000;
      ckdivm_reg <= 4'b0000;
      ckdivm_rst <= 1'b0;
      ckph1_pls  <= 1'b0;
      pwo_pls    <= 1'b0;
      end
    else begin
      ckdivc_pls <= ckdivc_en;
      ckdivl_reg <= {( !ckdivl_reg[4]   && ckdivl_reg[3]),
                     (~|ckdivl_reg[4:3] && ckdivl_reg[2]),
                     (~|ckdivl_reg[4:2] && ckdivl_reg[1]),
                      ~|ckdivl_reg[4:1]};
      ckdivm_reg <= (ckdivm_rst) ? 4'b0000  : (ckdivm_reg + ckdivl_reg[4]);
      ckdivm_rst <= ckdivl_reg[3] && ckdivm_reg[3] && |ckdivm_reg[2:0];
      ckph1_pls  <= ckdivl_reg[1] && ckdivm_reg[3] && !ckdivm_reg[0];
      pwo_pls    <= ckdivl_reg[4] && ~|(ckdivm_reg ^ 4'b0011) && statex_b && cyclex_reg[2];
      end
    end

  always @ (negedge clk or negedge resetb) begin
    if (!resetb) begin
      ph1 <= 1'b0;
      ph2 <= 1'b0;
      end
    else begin
      ph1 <= ((&ckdivm_reg[2:1] && ckdivl_reg[1]) ||
              (!(ckdivm_reg[3] && ckdivl_reg[3]) && ph1));
      ph2 <= ckdivm_rst || (!(ckdivm_reg[1] && ckdivl_reg[1]) && ph2);
      end
    end

  always @ (turbox_reg or ckdivl_reg or ckdivm_reg) begin
    case (turbox_reg)
      3'b001:  ckdivc_en = ckdivl_reg[3] && ((ckdivm_reg[2] && ~|ckdivm_reg[1:0]) ||
                                             (ckdivm_reg[3] && |ckdivm_reg[2:0]));
      3'b010:  ckdivc_en = ckdivl_reg[3] && ckdivm_reg[0];
      3'b011:  ckdivc_en = ckdivl_reg[3];
      3'b100:  ckdivc_en = ckdivl_reg[3] || ckdivl_reg[1];
      3'b101:  ckdivc_en = 1'b1;
      default: ckdivc_en = 1'b0;
      endcase
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* external bus state machine                                                            */
  /*                                                                                       */
  /*****************************************************************************************/
  /*                                                                                       */
  /*   statex_reg    eeeecccc88880000111122225555aaaa4444999933336666ddddbbbb              */
  /*   cyclex_reg    12481248124812481248124812481248124812481248124812481248              */
  /*   ld_cyclex     ...1...1...1...1...1...1...1...1...1...1...1...1...1...1              */
  /*   ld_instx      .......................................................1              */
  /*   isax_drv      ..............1111111111111111..........................              */
  /*   syncx_reg     ............................................1111111111..              */
  /*   tex_ky        ............11111111....................................              */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2x or negedge resetb) begin
    if (!resetb) cyclex_reg <= 4'b0001;
    else         cyclex_reg <= {( !cyclex_reg[3]   && cyclex_reg[2]),
                                (~|cyclex_reg[3:2] && cyclex_reg[1]),
                                 ~|cyclex_reg[3:1], cyclex_reg[3]};
    end

  always @ (posedge ph2x or negedge resetb) begin
    if        (!resetb) statex_reg <= 4'b1111;
    else if (ld_cyclex) statex_reg <= {(statex_reg[2] || statex_b), statex_reg[1:0],
                                       ((statex_reg[0] ^ !statex_reg[3]) && !statex_b)};
    end

  assign ld_cyclex = cyclex_reg[3];
  assign ld_instx  = statex_b && cyclex_reg[3];
  assign statex_b  = statex_reg[3] && &statex_reg[1:0];
  assign tex_ky    = ~|statex_reg[3:1];
  assign tex_a14   = ~|(statex_reg ^ 4'b1010) && cyclex_reg[0];

  /*****************************************************************************************/
  /*                                                                                       */
  /* external bus timing                                                                   */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2x or negedge resetb) begin
    if (!resetb) begin
      isax_drv  <= 1'b0;
      syncx_reg <= 1'b0;
      end
    else begin
      isax_drv  <= (~|statex_reg && cyclex_reg[1]) ||
                   (!(~|(statex_reg ^ 4'b1010) && cyclex_reg[1]) && isax_drv);
      syncx_reg <= (~|(statex_reg ^ 4'b0011) && cyclex_reg[3]) ||
                   (!(statex_b && cyclex_reg[1]) && syncx_reg);
      end
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* isa address & data latch                                                              */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge ph2c or negedge resetb) begin
    if      (!resetb) addr_smp <= 2'b00;
    else if (isa_drv) addr_smp <= {addr_smp[0], isa_out};
    end

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      isa_drv_sync  <= 1'b0;
      isax_drv_sync <= 1'b0;
      end
    else begin
      isa_drv_sync  <= isa_drv;
      isax_drv_sync <= isax_drv && trbref_reg;
      end
    end

  assign isa_drv_en = isa_drv_sync && clk2en;
  assign isa_recir  = isax_drv_sync && ckdivm_rst;

  always @ (posedge clk or negedge resetb) begin
    if                      (!resetb) addr_reg <= 16'h0000;
    else if (isa_drv_en || isa_recir) addr_reg <= (isa_drv_en) ?
                                                     {addr_smp[0], addr_reg[15:1]} :
                                                     {addr_reg[0], addr_reg[15:1]};
    end

  always @ (posedge ph2c or negedge resetb) begin
    if                     (!resetb) ftch_reg <= 10'h000;
    else if (sync_reg || sync_recir) ftch_reg <= (sync_reg) ? {isa_in,      ftch_reg[9:1]} :
                                                              {ftch_reg[0], ftch_reg[9:1]};
    end

  always @ (posedge ph2c or negedge resetb) begin
    if (!resetb) ld_inst_dly <= 1'b0;
    else         ld_inst_dly <= ld_inst;
    end

  assign page_012  = ~|laddr_reg[15:14] && ~&laddr_reg[13:12];
  assign page_35   = !laddr_reg[15] && laddr_reg[12] && (laddr_reg[14] ^ laddr_reg[13]);
  assign page_89   =  laddr_reg[15] && ~|laddr_reg[14:13];
  assign page_ab   =  laddr_reg[15] && !laddr_reg[14] && laddr_reg[13];
  assign page_cd   = &laddr_reg[15:14] && !laddr_reg[13];
  assign page_ef   = &laddr_reg[15:13];
  assign page_ext  = !mmmuen_reg && ( ~|(laddr_reg[15:12] ^ 4'b0100) ||
                                      ~|(laddr_reg[15:12] ^ 4'b0110) ||
                                     (~|(laddr_reg[15:12] ^ 4'b0111) && mmuen_global) ||
                                         laddr_reg[15]);
  assign page5_cur = te_a14 && !isa_nxt && isa_out && !addr_smp[0] && addr_smp[1];
  assign page5_ref = tex_a14 && !addr_reg[1] && addr_reg[0] && !addr_reg[15] && addr_reg[14];

  /*****************************************************************************************/
  /*                                                                                       */
  /* signals with clk timing                                                               */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clk or negedge resetb) begin
    if                 (!resetb) laddr_reg <= 4'b0000;
    else if (state_ac && clk2en) laddr_reg <= addr_reg[15:12];
    end

  always @ (posedge clk or negedge resetb) begin
    if                 (!resetb) lftch_reg <= 10'h000;
    else if (state_bc && clk2en) lftch_reg <= ftch_reg;
    end

  always @ (posedge clk) begin
    state_1c <= ld_cycle && ~|(state_reg ^ 4'b0001);
    state_4c <= ld_cycle && ~|(state_reg ^ 4'b0100);
    state_8c <= ld_cycle && ~|(state_reg ^ 4'b1000);
    state_9c <= ld_cycle && ~|(state_reg ^ 4'b1001);
    state_ac <= ld_cycle && ~|(state_reg ^ 4'b1010);
    state_bc <= ld_cycle && ~|(state_reg ^ 4'b1011);
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* special instruction decodes                                                           */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (lftch_reg) begin
    case (lftch_reg)
      10'b0000111000: cdata_inst = 1'b1;
      default:        cdata_inst = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    casex (lftch_reg) //synopsys parallel_case
      10'bxxx1111000,
      10'bxx1x111000,
      10'bx1xx111000,
      10'b1xxx111000: cregn_inst = 1'b1;
      default:        cregn_inst = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    case (lftch_reg)
      10'b1001110000: daddc_inst = 1'b1;
      default:        daddc_inst = 1'b0;
      endcase
    end

  always @ (minst_reg) begin
    casex (minst_reg)
      10'b0000111000,
      10'b1011110000: data_ftch = 1'b1;
      default:        data_ftch = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    casex (lftch_reg)
      10'b1011110000: datac_inst = 1'b1;
      default:        datac_inst = 1'b0;
      endcase
    end

  always @ (minst_reg) begin
    casex (minst_reg)
      10'b01xx000000: enbank_inst = 1'b1;
      default:        enbank_inst = 1'b0;
      endcase
    end

  always @ (minst_reg) begin
    casex (minst_reg)
      10'b1111001x00: kbrd_inst = 1'b1;                    /* rst kb, chk kb               */
      default:        kbrd_inst = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    casex (lftch_reg)
      10'b1111110000: pfadc_inst = 1'b1;
      default:        pfadc_inst = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    casex (lftch_reg[5:0])
      6'b101000: regnc_inst = 1'b1;
      default:   regnc_inst = 1'b0;
      endcase
    end

  always @ (minst_reg or r41_eany or r41_eblk) begin
    casex (minst_reg)
      10'bxxx1111000,
      10'bxx1x111000,
      10'bx1xx111000,
      10'b1xxx111000: rread_ftch = !r41_eblk;                   /* c=regn                  */
      10'b0000111000: rread_ftch = !r41_eany;                   /* c=data                  */
      default:        rread_ftch = 1'b0;
      endcase
    end

  always @ (minst_reg) begin
    casex (minst_reg)
      10'b1111001x00,                                      /* rst kb, chk kb               */
      10'b1001011000,                                      /* f=sb                         */
      10'b101x011000,                                      /* sb=f, fexsb                  */
      10'b10001x0000: slow_ftch = 1'b1;                    /* c=keys, gokeys               */
      default:        slow_ftch = 1'b0;
      endcase
    end

  always @ (minst_reg or r41_eany or r41_eblk) begin
    casex (minst_reg) //synopsys parallel_case
      10'bxxx1000000,                                      /* enromx, rsvd                 */
      10'bxx1x000000,                                      /* enromx, rsvd                 */
      10'bx1xx000000,                                      /* enromx, rsvd                 */
      10'b1xxx000000,                                      /* rsvd                         */
      10'b011100x100,                                      /* rsvd                         */
      10'b0111001x00,                                      /* rsvd                         */
      10'b0111010100,                                      /* rsvd                         */
      10'bxx00011000,                                      /* rsvd                         */
      10'b0111011100,                                      /* rsvd                         */
      10'b0001100000,                                      /* powoff                       */
      10'b1011100000,                                      /* disoff                       */
      10'b1100100000,                                      /* distog                       */
      10'bxxxx100100,                                      /* selprf                       */
      10'bxxxx101100,                                      /* ?fn=1                        */
      10'b0000110000,                                      /* rsvd                         */
      10'bx111110000,                                      /* rsvd                         */
      10'b1010110000,                                      /* clrdata                      */
      10'bxxxx110100,                                      /* rsvd                         */
      10'bx111111100: rftch_ftch = 1'b1;                   /* wcmd, pfad=c                 */
      10'bxxxx101000,                                      /* regn=c                       */
      10'bxxx1111000,
      10'bxx1x111000,
      10'bx1xx111000,
      10'b1xxx111000: rftch_ftch = r41_eblk;               /* c=regn                       */
      10'b0000111000,                                      /* c=data                       */
      10'b1011110000: rftch_ftch = r41_eany;               /* data=c                       */
      default:        rftch_ftch = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    case (lftch_reg)
      10'b0111111100: wcmd_inst = 1'b1;
      default:        wcmd_inst = 1'b0;
      endcase
    end

  always @ (lftch_reg) begin
    case (lftch_reg)
      10'b0001000000: wrom_inst = 1'b1;
      default:        wrom_inst = 1'b0;
      endcase
    end

  always @ (posedge ph2c or negedge resetb) begin
    if      (!resetb) slow_reg <= 3'b000;
    else if (ld_inst) slow_reg <= {(kbrd_inst && !jmpldi_reg),
                                   (kbrd_inst && !jmpldi_reg) || slow_reg[2],
                                   (kbrd_inst && !jmpldi_reg) || slow_reg[1]};
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* turbo register                                                                        */
  /*                                                                                       */
  /*****************************************************************************************/
  assign ld_turbo = wcmd_inst && dout_reg[19] && ~&dout_reg[18:17] && ld_inst_dly;

  always @ (posedge ph2c) begin
    if (ld_turbo || !dpwo) turbo_reg <= (ld_turbo) ? dout_reg[18:16] : 3'b000;
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* physical rom bank selects                                                             */
  /*                                                                                       */
  /*****************************************************************************************/
  assign clr_bank  = !dpwo || powoff_inst;
  assign ld_bank   = enbank_inst && ld_inst_dly;
  assign ld_35bank = ld_bank && page_35;
  assign ld_4bank  = ld_bank && !laddr_reg[15] && laddr_reg[14] && ~|laddr_reg[13:12];
  assign ld_6bank  = ld_bank && !laddr_reg[15] && &laddr_reg[14:13] && !laddr_reg[12];
  assign ld_7bank  = ld_bank && !laddr_reg[15] && &laddr_reg[14:12];
  assign ld_89bank = ld_bank && page_89;
  assign ld_abbank = ld_bank && page_ab;
  assign ld_cdbank = ld_bank && page_cd;
  assign ld_efbank = ld_bank && page_ef;

  always @ (posedge ph2c) begin
    if (ld_4bank  || clr_bank) bank4_reg  <= (ld_4bank)  ? lftch_reg[7:6] : 2'b00;
    if (ld_35bank || clr_bank) bank5_reg  <= (ld_35bank) ? lftch_reg[7:6] : 2'b00;
    if (ld_6bank  || clr_bank) bank6_reg  <= (ld_6bank)  ? lftch_reg[7:6] : 2'b00;
    if (ld_7bank  || clr_bank) bank7_reg  <= (ld_7bank)  ? lftch_reg[7:6] : 2'b00;
    if (ld_89bank || clr_bank) bank89_reg <= (ld_89bank) ? lftch_reg[7:6] : 2'b00;
    if (ld_abbank || clr_bank) bankab_reg <= (ld_abbank) ? lftch_reg[7:6] : 2'b00;
    if (ld_cdbank || clr_bank) bankcd_reg <= (ld_cdbank) ? lftch_reg[7:6] : 2'b00;
    if (ld_efbank || clr_bank) bankef_reg <= (ld_efbank) ? lftch_reg[7:6] : 2'b00;
    end

  always @ (laddr_reg or bank4_reg  or bank5_reg  or bank6_reg  or bank7_reg or
                         bank89_reg or bankab_reg or bankcd_reg or bankef_reg) begin
    casex (laddr_reg[15:12])
      4'b0100: bank_sel = bank4_reg;
      4'b0101: bank_sel = bank5_reg;
      4'b0110: bank_sel = bank6_reg;
      4'b0111: bank_sel = bank7_reg;
      4'b100x: bank_sel = bank89_reg;
      4'b101x: bank_sel = bankab_reg;
      4'b110x: bank_sel = bankcd_reg;
      4'b111x: bank_sel = bankef_reg;
      default: bank_sel = 2'b11;
      endcase
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* register address                                                                      */
  /*                                                                                       */
  /*****************************************************************************************/
  assign reg_emod = (cdata_inst || datac_inst) ? regadd_reg[3:0] : lftch_reg[9:6];
  assign reg_imod = (dftch_reg) ? regadd_reg[3:0] : diradd_reg;
  assign reg_addr = (rdtype_reg) ? {regadd_reg[11:4], reg_imod} :
                                   {regadd_reg[11:4], reg_emod};

  /*****************************************************************************************/
  /*                                                                                       */
  /* physical address and data                                                             */
  /*                                                                                       */
  /*****************************************************************************************/
  assign pg5_addr = (|bank5_reg) ? 12'h005 : 12'h006;

  always @ (laddr_reg or bank5_reg or mmmu_reg or mmmuen_reg or osmap_reg or pg5_addr) begin
    casex (laddr_reg[15:12])
      4'b00xx: trans_addr = (mmmuen_reg && osmap_reg) ? mmmu_reg : {8'h00, laddr_reg[15:12]};
      4'b0101: trans_addr = (mmmuen_reg && osmap_reg) ? mmmu_reg :  pg5_addr;
      default: trans_addr = (mmmuen_reg)              ? mmmu_reg : {8'h00, laddr_reg[15:12]};
      endcase
    end

  always @ (mphase_reg or addr_reg or bank_sel or dout_reg or laddr_reg or mmmu_reg or
            mword_reg or reg_addr or trans_addr or wrom_inst) begin
    casex (mphase_reg[5:1])
      5'bx0001: mem_addr = (wrom_inst) ? {16'h8040, dout_reg[27:24], 4'b0000} :
                                         {16'h8040, dout_reg[39:36], dout_reg[21:20], 2'b00};
      5'bx001x: mem_addr = {trans_addr, addr_reg[11:0]};
      5'bxx1xx: mem_addr = (|mword_reg[1:0]) ?  24'h804000 :
                                               {10'h200, reg_addr, |mword_reg[5:4],
                                                (mword_reg[5] || mword_reg[3])};
      5'b01000: mem_addr = dout_reg[47:24];
      5'b01001: mem_addr = {16'h8040, dout_reg[39:36], dout_reg[21:20], 2'b00};
      5'b0101x: mem_addr = (wrom_inst) ? {mmmu_reg, dout_reg[23:12]} :
                                         {mmmu_reg, dout_reg[35:24]};
      5'b110xx: mem_addr = 24'h804000;
      default:  mem_addr = {16'h8040, laddr_reg[15:12], bank_sel, 2'b00};
      endcase
    end

  always @ (mphase_reg or mword_reg or dout_reg or lftch_reg or minst_reg or regadd_reg or
            regnc_inst or rread_ftch or wrom_inst) begin
    casex ({mphase_reg[4:3], mword_reg[5:1]})
      7'bx11xxxx: mwr_data = {8'h00, dout_reg[55:48]};
      7'bx1x1xxx: mwr_data = dout_reg[47:32];
      7'bx1xx1xx: mwr_data = dout_reg[31:16];
      7'b01xxxx1: mwr_data = (rread_ftch && |minst_reg[9:6]) ?
                                            {4'h0, regadd_reg[11:4], minst_reg[9:6]} :
                                            {4'h0, regadd_reg};
      7'b11xxxx1: mwr_data = (regnc_inst && |lftch_reg[9:6]) ?
                                            {4'h0, regadd_reg[11:4], lftch_reg[9:6]} :
                                            {4'h0, regadd_reg};
      default:    mwr_data = (wrom_inst)  ? {4'h0, dout_reg[11:0]} : dout_reg[15:0];
      endcase
    end

  /*****************************************************************************************/
  /*                                                                                       */
  /* physical memory interface                                                             */
  /*                                                                                       */
  /*****************************************************************************************/
  /*                                                                                       */
  /*  mphase_reg                     mcycle_reg                    mword_reg               */
  /*  0000000  idle                  00  addr/data setup           000001  1/6             */
  /*  0000001  rd mmui (1)           01  rd/wr                     000010  2/6             */
  /*  0000010  rd mmu  (1)           11  rd/wr                     000100  3/6             */
  /*  0000100  rd inst (1)           10  addr/data hold            001000  4/6             */
  /*  0001000  rd reg  (6)                                         010000  5/6             */
  /*  0010001  wr phy  (1)                                         100000  6/6 or 1/1      */
  /*  1010001  rd phy  (1)                                                                 */
  /*  0010010  wr mmu  (1)                                                                 */
  /*  1010010  rd mmu  (1)                                                                 */
  /*  0010100  wr log  (1)                                                                 */
  /*  1010100  rd log  (1)                                                                 */
  /*  0011000  wr reg  (6)                                                                 */
  /*  0110000  wr add  (1)                                                                 */
  /*                                                                                       */
  /*****************************************************************************************/
  assign trig_wradd  = ld_instc_dly && daddc_inst && !selprf_reg;
  assign trig_erdreg = ld_instc_dly && reg_read && !rdtype_reg;
  assign trig_irdreg = state_9c && rread_ftch && !jmpldi_reg && !selprf_reg;
  assign trig_wrreg  = ld_instc_dly && reg_write;
  assign trig_rwmmu  = state_8c && wcmd_inst && ~|dout_reg[19:17] && |dout_reg[39:38];
  assign trig_rdmmu  = state_8c && ((wcmd_inst && ~|(dout_reg[19:17] ^ 3'b001)) ||
                                     wrom_inst);
  assign trig_rwlog  = state_1c && (wcmd_inst && ~|(dout_reg[19:17] ^ 3'b001)) && mmmuen_reg;
  assign trig_wrom   = state_1c && wrom_inst && mmmuen_reg;
  assign trig_rwphy  = state_1c && wcmd_inst && ~|(dout_reg[19:17] ^ 3'b010);
  assign reg_access  = trig_wrreg || trig_erdreg || trig_irdreg;

  assign m_strt      = trig_wradd || trig_erdreg || trig_wrreg || (clk2en &&
                       (state_4c || state_ac || trig_rwmmu || trig_wrom ||
                        trig_rdmmu || trig_irdreg || trig_rwlog || trig_rwphy));

  assign mword_nxt   = {!reg_access, 4'b0000, reg_access};
  assign mphase_nxt  = {((trig_rwmmu || trig_rwlog || trig_rwphy) && dout_reg[16]),
                         trig_wradd,
                        (trig_wradd || trig_rwphy || trig_rwmmu || trig_rwlog ||
                         trig_wrreg || trig_wrom),
                        (trig_erdreg || trig_irdreg || trig_wrreg),
                        (state_4c   || trig_rwlog || trig_wrom),
                        (trig_rdmmu || trig_rwmmu), (state_ac   || trig_rwphy)};

  assign ld_mword    = m_strt || (mcycle_reg[1] && !mcycle_reg[0]);
  assign ld_mphase   = m_strt || (mword_reg[5] && mcycle_reg[1] && !mcycle_reg[0]);

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      dftch_reg  <= 1'b0;
      diradd_reg <= 4'b0000;
      rdtype_reg <= 1'b0;
      end
    else if (state_9c) begin
      dftch_reg  <= data_ftch;
      diradd_reg <= minst_reg[9:6];
      rdtype_reg <= rread_ftch && !jmpldi_reg && !selprf_reg;
      end
    end

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) mcycle_reg <= 2'b00;
    else         mcycle_reg <= {mcycle_reg[0], (|mphase_reg[4:0] && !mcycle_reg[1])};
    end

  always @ (posedge clk or negedge resetb) begin
    if       (!resetb) mword_reg <= 6'b000000;
    else if (ld_mword) mword_reg <= (m_strt) ? mword_nxt : {mword_reg[4:0], 1'b0};
    end

  always @ (posedge clk or negedge resetb) begin
    if        (!resetb) mphase_reg <= 7'b0000000;
    else if (ld_mphase) mphase_reg <= (!m_strt && mword_reg[3]) ? 7'b0000000 : mphase_nxt;
    end

  assign mem_idl = ~|mphase_reg[4:0];
  assign mem_wr  = (!mphase_reg[6] && mphase_reg[4] && !mword_reg[0]) || mword_reg[1];

  assign MEMCS0B = !(|mphase_reg[4:0] && !mem_addr[23]);
  assign mem_cs1 =   |mphase_reg[4:0] &&  mem_addr[23];
  assign MEMRDB  = !(((!mphase_reg[4] && !mword_reg[1]) || mword_reg[0] || mphase_reg[6]) &&
                        mcycle_reg[0]);
  assign MEMWRB  = !(mem_wr && mcycle_reg[0]);

  /*****************************************************************************************/
  /*                                                                                       */
  /* instruction, register & mmu data from physical memory                                 */
  /*                                                                                       */
  /*****************************************************************************************/
  assign rd_mdata   = !mphase_reg[4] && mphase_reg[3] && &mcycle_reg;

  assign uld_regadd = &mphase_reg[5:4] && &mcycle_reg;
  assign urd_regadd = cregn_inst && !jmpldi_reg && !selprf_reg && ld_instc_dly;
  assign uwr_regadd = regnc_inst && &mphase_reg[4:3] && mword_reg[1] && &mcycle_reg;
  assign cld_regadd = uld_regadd || urd_regadd || uwr_regadd;

  assign ld_ldata   =  mphase_reg[6] && &mcycle_reg;
  assign ld_mdata0  = (rd_mdata && mword_reg[2]) || mdata0_shft;
  assign ld_mdata1  = (rd_mdata && mword_reg[3]) || mdata1_shft;
  assign ld_mdata2  = (rd_mdata && mword_reg[4]) || mdata2_shft;
  assign ld_mdata3  = (rd_mdata && mword_reg[5]) || mdata3_shft;
  assign ld_minst   = !mphase_reg[4] && mphase_reg[2] && &mcycle_reg;
  assign ld_mmmu    = !mphase_reg[4] && |mphase_reg[1:0] && &mcycle_reg;
  assign minst_shft = sync_rawc && clk2en;

  assign ld_mmudis = wcmd_inst && ~|(dout_reg[19:16] ^ 4'b0110) && ld_instc_dly;
  assign ld_mmuen  = wcmd_inst && ~|(dout_reg[19:16] ^ 4'b0111) && ld_instc_dly;
  assign ld_mmucp  = page_012 && ld_instc_dly;
  assign ld_mosdis = wcmd_inst && ~|(dout_reg[19:16] ^ 4'b1110) && ld_instc_dly;
  assign ld_mosen  = wcmd_inst && ~|(dout_reg[19:16] ^ 4'b1111) && ld_instc_dly;

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      lmmudis_reg <= 1'b0;
      lmmuen_reg  <= 1'b0;
      wr_mmudis   <= 1'b0;
      wr_mmuen    <= 1'b0;
      end
    else begin
      lmmudis_reg <= !ld_mmuen  && !ld_mmucp && (ld_mmudis || lmmudis_reg);
      lmmuen_reg  <= !ld_mmudis && !ld_mmucp && (ld_mmuen  || lmmuen_reg);
      wr_mmudis   <= lmmudis_reg && ld_mmucp;
      wr_mmuen    <= lmmuen_reg  && ld_mmucp;
      end
    end

  always @ (posedge clk) begin
    osmap_reg <= dpwo && !ld_mosdis && (ld_mosen || osmap_reg);
    end

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) regadd_reg <= 12'h000;
    else begin
      if (uld_regadd) regadd_reg[11:4] <= mwr_data[11:4];
      if (cld_regadd) regadd_reg[3:0]  <= (urd_regadd) ? diradd_reg : mwr_data[3:0];
      end
    end

  always @ (posedge clk) begin
    if (ld_mdata0) mreg0_reg <= (mdata0_shft) ? {mreg0_reg[0], mreg0_reg[15:1]} : mrd_data;
    if (ld_mdata1) mreg1_reg <= (mdata1_shft) ? {mreg1_reg[0], mreg1_reg[15:1]} : mrd_data;
    if (ld_mdata2) mreg2_reg <= (mdata2_shft) ? {mreg2_reg[0], mreg2_reg[15:1]} : mrd_data;
    if (ld_mdata3) mreg3_reg <= (mdata3_shft) ? {mreg3_reg[0], mreg3_reg[7:1]} :
                                                 mrd_data[7:0];
    end

  always @ (posedge clk or negedge resetb) begin
    if                     (!resetb) minst_reg <= 10'h000;
    else if (ld_minst || minst_shft) minst_reg <= (minst_shft) ?
                                                           {minst_reg[0], minst_reg[9:1]} :
                                                  (page_ext)   ? 10'h000 : mrd_data[9:0];
    end

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      ldata_reg  <= 16'h0000;
      mspeci_reg <= 2'b01;
      end
    else begin
      if (ld_ldata) ldata_reg  <= mrd_data;
      if (ld_minst) mspeci_reg <= mrd_data[13:12];
      end
    end

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      mmmu_reg   <= 12'h000;
      mmmuen_reg <= 1'b0;
      end
    else if (ld_mmmu) begin
      mmmu_reg   <= mrd_data[11:0];
      mmmuen_reg <= mmuen_global && mrd_data[15];
      end
    end

  always @ (state_regc) begin
    case (state_regc)
      4'b1110,
      4'b1100,
      4'b1000,
      4'b0000: mword_sel = 4'b0001;
      4'b0001,
      4'b0010,
      4'b0101,
      4'b1010: mword_sel = 4'b0010;
      4'b0100,
      4'b1001,
      4'b0011,
      4'b0110: mword_sel = 4'b0100;
      default: mword_sel = 4'b1000;
      endcase
    end

  assign mreg_out = (mword_sel[0] && mreg0_reg[0]) || (mword_sel[1] && mreg1_reg[0]) ||
                    (mword_sel[2] && mreg2_reg[0]) || (mword_sel[3] && mreg3_reg[0]);

  /*****************************************************************************************/
  /*                                                                                       */
  /* peripheral interface for uart                                                         */
  /*                                                                                       */
  /*****************************************************************************************/
  assign ld_pfad  =  ld_instc_dly && pfadc_inst;
  assign ld_pdata = (ld_instc_dly && cregn_inst) || mdata_shft;

  always @ (posedge clk or negedge resetb) begin
    if      (!resetb) pfads_reg <= 1'b0;
    else if (ld_pfad) pfads_reg <= &dout_reg[7:4] && ~|dout_reg[3:0];
    end

  always @ (posedge clk) begin
    if (ld_pdata) preg_reg <= (mdata_shft) ? {1'b0, preg_reg[15:1]} : uart_data;
    end

  assign ld_divrate = pfads_reg && ld_instc_dly && regnc_inst &&
                      &lftch_reg[9:8] && ~|lftch_reg[7:6];
  assign ld_parport = pfads_reg && ld_instc_dly && regnc_inst &&
                      lftch_reg[9] && ~|lftch_reg[8:6];
  assign ld_usctrl  = pfads_reg && ld_instc_dly && regnc_inst &&
                      lftch_reg[9] && !lftch_reg[8] && &lftch_reg[7:6];
  assign us_bufr_rd = pfads_reg && ld_instc_dly && cregn_inst && &lftch_reg[9:6];
  assign us_bufr_wr = pfads_reg && ld_instc_dly && regnc_inst && &lftch_reg[9:6];

  always @ (posedge clk or negedge resetb) begin
    if (!resetb) begin
      par_dpls <= 1'b0;
      s_reset  <= 1'b0;
      end
    else begin
      par_dpls <= ld_parport ^ par_dpls;
      s_reset  <= (pfads_reg && ld_instc_dly && regnc_inst && lftch_reg[9] &&
                   !lftch_reg[8] && lftch_reg[7] && !lftch_reg[6] && dout_reg[0]) ||
                  (!dpwo && ckph1_pls);
      end
    end

  always @ (posedge clk) begin
    if (ld_divrate || s_reset) div_rate <= (s_reset) ? 12'hfff : dout_reg[11:0];
    if (ld_usctrl  || s_reset) s_wp     <=  s_reset || dout_reg[8];
    if (ld_usctrl  || s_reset) s_ce     <= !s_reset && dout_reg[4];
    if (ld_usctrl  || s_reset) cks_mode <= !s_reset && dout_reg[0];
    end

  always @ (pfads_reg or lftch_reg or ldata_reg or mmuen_global or turbo_reg or
            us_bufr_full or us_bufr_done or us_bufr_empt or us_bufr_ovr or us_data_out) begin
    case ({pfads_reg, lftch_reg[9:6]})
      5'b11111: uart_data = {8'h00, us_data_out};
      5'b11110: uart_data = {8'h00, 3'b000, us_bufr_ovr,  3'b000, us_bufr_full}; 
      5'b11101: uart_data = {8'h00, 3'b000, us_bufr_done, 3'b000, us_bufr_empt};
      5'b10110: uart_data = {8'h00, 7'b0000000, mmuen_global};
      5'b10100: uart_data = {8'h00, 5'b00000, turbo_reg};
      5'b10010: uart_data = ldata_reg;
      default:  uart_data = 16'h0000;
      endcase
    end

  sio_top UART  ( .s_clk_out(s_clk_out), .s_data_out(s_data_out),
                  .us_bufr_done(us_bufr_done), .us_bufr_empt(us_bufr_empt),
                  .us_bufr_full(us_bufr_full), .us_bufr_ovr(us_bufr_ovr), 
                  .us_data_out(us_data_out), .u_data_out(u_data_out), .clkp(clk),
                  .cks_mode(cks_mode), .div_rate(div_rate), .s_data_in(s_data_in),
                  .s_reset(s_reset), .us_bufr_rd(us_bufr_rd), .us_bufr_wr(us_bufr_wr),
                  .us_data_in(dout_reg[7:0]), .u_data_in(u_data_in) );

  /*****************************************************************************************/
  /*                                                                                       */
  /* nut processor                                                                         */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clk) begin
    chkkb_instc  <= chkkb_inst;
    powoff_instc <= powoff_inst;
    rstkb_instc  <= rstkb_inst;
    state_regc   <= state_reg;
    sync_rawc    <= sync_raw;
    end

  nut_cpu CPU   ( .chkkb_inst(chkkb_inst), .cycle_reg(cycle_reg), .data_drv(data_drv),
                  .data_out(data_out), .dout_reg(dout_reg), .fo_bus(fo_bus),
                  .isa_drv(isa_drv), .isa_nxt(isa_nxt), .isa_out(isa_out),
                  .jmpldi_reg(jmpldi_reg), .ld_cycle(ld_cycle), .ld_inst(ld_inst),
                  .powoff_inst(powoff_inst), .rstkb_inst(rstkb_inst),
                  .selprf_reg(selprf_reg), .state_reg(state_reg), .sync_raw(sync_raw),
                  .sync_reg(sync_reg), .te_a14(te_a14), .data_in(data_in), .dpwo(dpwo),
                  .fi_bus(fi_bus), .isa_in(isa_in), .ky_data(ky_datac), .ky_flag(ky_flagc),
                  .lld(lld), .ph2(ph2c), .rstb(resetb) );

  /*****************************************************************************************/
  /*                                                                                       */
  /* nut keyboard scanner                                                                  */
  /*                                                                                       */
  /*****************************************************************************************/
  always @ (posedge clk) begin
    ky_datac   <= ky_data;
    ky_flagc   <= ky_flag;
    end

  kbrd_top KBRD ( .kc0(kc0), .kc1(kc1), .kc2(kc2), .kc3(kc3), .kc4(kc4), .kc5(kc5),
                  .kc6(kc6), .ky_data(ky_data), .ky_flag(ky_flag), .chkkb_inst(chkkb_instc),
                  .kr0(kr0), .kr1(kr1), .kr2(kr2), .kr3(kr3), .kr4(kr4), .kr5(kr5),
                  .kr6(kr6), .kr7(kr7), .ld_cycle(ld_cyclex), .ld_inst(ld_instx),
                  .ph2(ph2x), .por(por), .rstb(resetb), .rstkb_inst(rstkb_instc),
                  .te_ky(tex_ky) );

  /*****************************************************************************************/
  /*                                                                                       */
  /* FPGA-specific: FlashROM disable                                                       */
  /*                                                                                       */
  /*****************************************************************************************/
  ULSICC U1 ( .LSICC(1'b1) ) /* synthesis syn_noprune=1 */;

  endmodule




















